@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -28,7 +28,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -38,7 +38,9 @@ from . import amqp
 from . import api
 from . import asynchronous
 from . import base
+from . import bus
 from . import cache
+from . import component
 from . import compress
 from . import config
 from . import controller
@@ -46,6 +48,7 @@ from . import crypt
 from . import data
 from . import defines
 from . import exceptions
+from . import execution
 from . import export
 from . import extra
 from . import geo
@@ -59,6 +62,7 @@ from . import model
 from . import mongo
 from . import observer
 from . import part
+from . import preferences
 from . import queuing
 from . import redisdb
 from . import request
@@ -81,31 +85,38 @@ from .asynchronous import ASYNC_HEADER, AsyncManager, SimpleManager, QueueManage
     ensure_async, header_a, ensure_a
 from .base import APP, LEVEL, NAME, VERSION, PLATFORM, IDENTIFIER_SHORT, IDENTIFIER_LONG, IDENTIFIER,\
     API_VERSION, BUFFER_SIZE, MAX_LOG_SIZE, MAX_LOG_COUNT, App, APIApp, WebApp, Template, get_app, get_name,\
-    get_base_path, get_request, get_session, get_model, get_controller, get_part, get_adapter, get_manager,\
-    get_logger, get_level, is_devel, is_safe, to_locale, on_exit
-from .cache import Cache, MemoryCache, FileCache, RedisCache
+    get_base_path, get_cache, get_preferences, get_bus, get_request, get_session, get_model, get_controller, get_part,\
+    get_adapter, get_manager, get_logger, get_level, is_loaded, is_devel, is_safe, to_locale, on_exit
+from .bus import Bus, MemoryBus, RedisBus
+from .cache import Cache, MemoryCache, FileCache, RedisCache, SerializedCache
+from .component import Component
 from .compress import Compress
-from .config import conf, conf_prefix, conf_suffix, conf_s, conf_d
+from .config import conf, conf_prefix, conf_suffix, conf_s, conf_r, conf_d
 from .controller import Controller
 from .crypt import Cipher, RC4, Spritz
 from .data import DataAdapter, MongoAdapter, TinyAdapter, Collection, MongoCollection, TinyCollection
 from .defines import ITERABLES, MOBILE_REGEX, TABLET_REGEX, MOBILE_PREFIX_REGEX, BODY_REGEX, TAG_REGEX,\
-    EMAIL_REGEX, WINDOWS_LOCALE, SLUG_PERMUTATIONS
+    EMAIL_REGEX, BROWSER_INFO, OS_INFO, WINDOWS_LOCALE, SLUG_PERMUTATIONS
 from .exceptions import AppierException, OperationalError, SecurityError, AssertionError,\
     ValidationError, NotFoundError, NotImplementedError, BaseInternalError, ValidationInternalError,\
     ValidationMultipleError, HTTPError, APIError, APIAccessError, OAuthAccessError
+from .execution import ExecutionThread, background, insert_work, interval_work, seconds_work,\
+    minutes_work, hourly_work, daily_work, weekly_work, monthly_work, seconds_eval, minutes_eval,\
+    hourly_eval, daily_eval, weekly_eval, monthly_eval
 from .export import ExportManager
 from .extra import get_a, post_a, put_a, delete_a, patch_a, get_w, post_w, put_w, delete_w, patch_w
 from .geo import GeoResolver
 from .git import Git
 from .http import get_f, get, post, put, delete, HTTPResponse
-from .log import MemoryHandler, ThreadFormatter, DummyLogger, rotating_handler, smtp_handler, in_signature
+from .log import MemoryHandler, BaseFormatter, ThreadFormatter, DummyLogger, rotating_handler, smtp_handler,\
+    in_signature
 from .meta import Ordered, Indexed
 from .mock import MockObject, MockResponse, MockApp
 from .model import Model, LocalModel, Field, link, operation, view, field
 from .mongo import Mongo, MongoEncoder, get_connection, reset_connection, get_db, drop_db, object_id, dumps
 from .observer import Observable
 from .part import Part
+from .preferences import Preferences, MemoryPreferences, FilePreferences, RedisPreferences
 from .queuing import Queue, MemoryQueue, MultiprocessQueue, AMQPQueue
 from .redisdb import Redis
 from .request import CODE_STRINGS, Request, MockRequest
@@ -119,14 +130,15 @@ from .storage import StorageEngine, BaseEngine, FsEngine
 from .structures import OrderedDict, LazyDict, LazyValue, lazy_dict, lazy
 from .typesf import AbstractType, Type, File, Files, ImageFile, ImageFiles, image, images, Reference,\
     reference, References, references, Encrypted, encrypted, secure
-from .util import is_iterable, is_mobile, is_tablet, email_parts, email_mime, email_name, email_base,\
-    date_to_timestamp, obfuscate, import_pip, ensure_pip, install_pip, install_pip_s, request_json,\
-    get_object, resolve_alias, page_types, find_types, norm_object, set_object, leafs, gather_errors, gen_token,\
-    html_to_text, camel_to_underscore, camel_to_readable, underscore_to_readable, quote, unquote,\
-    call_safe, base_name, base_name_m, is_content_type, parse_content_type, parse_cookie, parse_multipart,\
+from .util import is_iterable, is_mobile, is_tablet, is_browser, is_bot, browser_info, email_parts, email_mime,\
+    email_name, email_base, date_to_timestamp, obfuscate, import_pip, ensure_pip, install_pip, install_pip_s,\
+    request_json, get_context, get_object, resolve_alias, page_types, find_types, norm_object, set_object, leafs,\
+    gather_errors, gen_token, html_to_text, camel_to_underscore, camel_to_readable, underscore_to_readable, quote,\
+    unquote, call_safe, base_name, base_name_m, is_content_type, parse_content_type, parse_cookie, parse_multipart,\
     decode_params, load_form, check_login, check_token, check_tokens, ensure_login, get_tokens_m, to_tokens_m,\
     dict_merge, cached, private, ensure, delayed, route, error_handler, exception_handler, before_request,\
-    after_request, is_detached, sanitize, verify, execute, ctx_locale, ctx_request, FileTuple, BaseThread, JSONEncoder
+    after_request, is_detached, sanitize, verify, verify_equal, verify_not_equal, execute, ctx_locale, ctx_request,\
+    FileTuple, BaseThread, JSONEncoder
 from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null,\
     not_empty, not_false, is_in, is_lower, is_simple, is_email, is_url, is_regex, field_eq, field_gt,\
     field_gte, field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
