@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -106,8 +106,7 @@ class AssertionError(OperationalError):
 
     def __init__(self, message = None, code = None):
         OperationalError.__init__(self,
-            message = message or "Assertion of data failed",
-            code = code or 400
+            message = message or "Assertion of data failed"
         )
 
 class ValidationError(OperationalError):
@@ -279,7 +278,7 @@ class HTTPError(BaseInternalError):
             data = self.read(error = error)
             try: data = data.decode("utf-8")
             except: data = legacy.str(data)
-            if data: message += "\n" + data
+            if data: message = message + "\n" + data if message else data
         BaseInternalError.__init__(self, message)
         self.code = code
         self.error = error
