@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -431,7 +431,8 @@ def _method_empty(
         handle = handle,
         silent = silent,
         redirect = redirect,
-        timeout = timeout
+        timeout = timeout,
+        **kwargs
     )
 
     if not silent: logging.debug("%s %s returned '%d'" % (name, url, code))
@@ -555,16 +556,19 @@ def _redirect(
     handle = None,
     silent = None,
     redirect = None,
-    timeout = None
+    timeout = None,
+    **kwargs
 ):
     is_relative = location.startswith("/")
     if is_relative: location = scheme + "://" + host + location
+    logging.debug("Redirecting to %s" % location)
     return get(
         location,
         handle = handle,
         silent = silent,
         redirect = redirect,
-        timeout = timeout
+        timeout = timeout,
+        **kwargs
     )
 
 def _resolve(*args, **kwargs):
@@ -682,17 +686,31 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
     retry = kwargs.get("retry", 1)
     reuse = kwargs.get("reuse", True)
     level = kwargs.get("level", level)
-    async = kwargs.get("async", False)
+    asynchronous = kwargs.get("async", False)
+    asynchronous = kwargs.get("asynchronous", asynchronous)
+    use_file = kwargs.get("use_file", False)
     callback = kwargs.get("callback", None)
+    callback_headers = kwargs.get("callback_headers", None)
+    callback_data = kwargs.get("callback_data", None)
+    callback_result = kwargs.get("callback_result", None)
 
     # re-calculates the retry and re-use flags taking into account
     # the async flag, if the execution mode is async we don't want
     # to re-use the HTTP client as it would create issues
-    retry, reuse = (0, False) if async else (retry, reuse)
+    retry, reuse = (0, False) if asynchronous else (retry, reuse)
 
     # creates the proper set of extra parameters to be sent to the
     # HTTP client taking into account a possible async method request
-    extra = _async_netius(callback) if async else dict()
+    extra = _async_netius(
+        callback = callback,
+        callback_headers = callback_headers,
+        callback_data = callback_data,
+        callback_result = callback_result
+    ) if asynchronous else dict(
+        on_headers = lambda c, p: callback_headers and callback_headers(p.headers),
+        on_data = lambda c, p, d: callback_data and callback_data(d),
+        on_result = lambda c, p, r: callback_result and callback_result(r)
+    )
 
     # verifies if client re-usage must be enforced and if that's the
     # case the global client object is requested (singleton) otherwise
@@ -703,8 +721,9 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
         url,
         headers = headers,
         data = data,
-        async = async,
+        asynchronous = asynchronous,
         timeout = timeout,
+        use_file = use_file,
         http_client = http_client,
         level = level,
         **extra
@@ -712,7 +731,7 @@ def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
 
     # if the async mode is defined an invalid value is returned immediately
     # as the processing will be taking place latter (on callback)
-    if async: return None
+    if asynchronous: return None
 
     # tries to retrieve any possible error coming from the result object
     # if this happens it means an exception has been raised internally and
@@ -765,6 +784,10 @@ def _client_netius(level = logging.CRITICAL):
     finally:
         ACCESS_LOCK.release()
 
+    # in case the retrieved client is not valid (stopped) then
+    # it must be invalidated as it cannot be re-used
+    if netius_client and netius_client.is_stopped(): netius_client = None
+
     # in case a previously created netius client has been retrieved
     # returns it to the caller method for proper re-usage
     if netius_client: return netius_client
@@ -784,7 +807,12 @@ def _client_netius(level = logging.CRITICAL):
     if not registered: common.base().on_exit(_cleanup_netius)
     return netius_client
 
-def _async_netius(callback):
+def _async_netius(
+    callback = None,
+    callback_headers = None,
+    callback_data = None,
+    callback_result = None
+):
     import netius.clients
 
     buffer = []
@@ -793,9 +821,16 @@ def _async_netius(callback):
     def _on_close(connection):
         callback and callback(None)
 
+    def _on_headers(client, parser):
+        callback_headers and callback_headers(parser.headers)
+
     def _on_data(client, parser, data):
         data = data
         data and buffer.append(data)
+        callback_data and callback_data(data)
+
+    def _on_result(client, parser, result):
+        callback_result and callback_result(result)
 
     def _callback(connection, parser, message):
         result = netius.clients.HTTPClient.set_request(parser, buffer)
@@ -805,6 +840,8 @@ def _async_netius(callback):
     extra["callback"] = _callback
     extra["on_close"] = _on_close
     extra["on_data"] = _on_data
+    if callback_headers: extra["on_headers"] = _on_headers
+    if callback_result: extra["on_result"] = _on_result
 
     return extra
 
@@ -883,14 +920,18 @@ def _params(query):
     return params
 
 def _urlencode(values, as_string = True):
-    # creates the dictionary that will hold the final
-    # dictionary of values (without the unset and
-    # invalid values)
-    final = dict()
+    # creates the list that will hold the final tuple of values
+    # (without the unset and invalid values)
+    final = []
 
-    # iterates over all the items in the values map to
+    # verifies if the provided value is a sequence and in case it's
+    # not converts it into a sequence (assuming a map)
+    is_sequence = isinstance(values, (list, tuple))
+    if not is_sequence: values = values.items()
+
+    # iterates over all the items in the values sequence to
     # try to filter the values that are not valid
-    for key, value in values.items():
+    for key, value in values:
         # creates the list that will hold the valid values
         # of the current key in iteration (sanitized values)
         _values = []
@@ -922,7 +963,7 @@ def _urlencode(values, as_string = True):
 
         # sets the sanitized list of values as the new value for
         # the key in the final dictionary of values
-        final[key] = _values
+        final.append((key, _values))
 
     # in case the "as string" flag is not set the ended key to value
     # dictionary should be returned to the called method and not the
@@ -968,6 +1009,7 @@ def _encode_multipart(fields, mime = None, doseq = False):
         values = values if is_list else [values]
 
         for value in values:
+            if value == None: continue
 
             if isinstance(value, dict):
                 header_l = []
@@ -1022,7 +1064,9 @@ def _try_boundary(fields, boundary, doseq = False):
 
         for value in values:
             if isinstance(value, dict): name = ""; value = value.get("data", b"")
-            elif isinstance(value, tuple): name = value[0] or ""; value = value[1]
+            elif isinstance(value, tuple):
+                if len(value) == 2: name = value[0] or ""; value = value[1] or b""
+                else: name = value[0] or ""; value = value[2] or b""
             else: name = ""; value = _encode(value)
 
             if not key.find(boundary) == -1: return False
