@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -75,6 +75,9 @@ class Indexed(type):
         new_cls = super(Indexed, cls).__new__(cls, name, bases, attrs)
         new_name = new_cls.__name__
 
+        # retrieves the complete set of elements in the current set of attributes
+        # that are marked with the "creation counter" and then sorts these same
+        # elements according to that value (sorting by order of definition)
         ordered = [(name, attrs.pop(name)) for name, value in\
             legacy.eager(attrs.items()) if hasattr(value, "creation_counter")]
         ordered.sort(key = lambda item: item[1].creation_counter)
@@ -109,7 +112,7 @@ class Indexed(type):
                 # retrieves the method reference associated with the function, this
                 # is required as the current reference is just an unbound function
                 # and the bound method is required for registration
-                url, method, async, json = route
+                url, method, asynchronous, json = route
                 function = getattr(new_cls, name)
 
                 # creates the tuple that identifies the route as a set
@@ -127,7 +130,7 @@ class Indexed(type):
                     method,
                     url,
                     function,
-                    async = async,
+                    asynchronous = asynchronous,
                     json = json,
                     context = new_name
                 )
