@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -39,10 +39,12 @@ __license__ = "Apache License, Version 2.0"
 
 import json
 
+from . import util
 from . import legacy
 from . import typesf
 from . import config
 from . import common
+from . import exceptions
 
 try: import pymongo
 except: pymongo = None
@@ -71,8 +73,8 @@ class Mongo(object):
         url_c = config.conf("MONGOLAB_URI", url_c)
         url_c = config.conf("MONGO_URL", url_c)
         url = url or self.url or url_c or URL
-        if is_new(): self._connection = pymongo.MongoClient(url, connect = connect)
-        else: self._connection = pymongo.Connection(url)
+        if is_new(): self._connection = _pymongo().MongoClient(url, connect = connect)
+        else: self._connection = _pymongo().Connection(url)
         return self._connection
 
     def reset_connection(self):
@@ -101,8 +103,8 @@ def get_connection(url = URL, connect = False):
     url = config.conf("MONGOHQ_URL", url)
     url = config.conf("MONGOLAB_URI", url)
     url = config.conf("MONGO_URL", url)
-    if is_new(): connection = pymongo.MongoClient(url, connect = connect)
-    else: connection = pymongo.Connection(url)
+    if is_new(): connection = _pymongo().MongoClient(url, connect = connect)
+    else: connection = _pymongo().Connection(url)
     return connection
 
 def reset_connection():
@@ -146,12 +148,12 @@ def serialize(obj):
 
 def directions(all = False):
     return (
-        pymongo.ASCENDING,
-        pymongo.DESCENDING,
-        pymongo.HASHED
+        _pymongo().ASCENDING,
+        _pymongo().DESCENDING,
+        _pymongo().HASHED
     ) if all else (
-        pymongo.ASCENDING,
-        pymongo.DESCENDING
+        _pymongo().ASCENDING,
+        _pymongo().DESCENDING
     )
 
 def is_mongo(obj):
@@ -160,7 +162,7 @@ def is_mongo(obj):
     return False
 
 def is_new():
-    return int(pymongo.version[0]) >= 3 if pymongo else False
+    return int(_pymongo().version[0]) >= 3 if pymongo else False
 
 def _store_find_and_modify(store, *args, **kwargs):
     if is_new(): store.find_one_and_update(*args, **kwargs)
@@ -191,3 +193,11 @@ def _store_ensure_index_many(store, *args, **kwargs):
         _args = list(args)
         _args[0] = [(_args[0], direction)]
         _store_ensure_index(store, *_args, **kwargs)
+
+def _pymongo(verify = True):
+    if verify: util.verify(
+        not pymongo == None,
+        message = "pymongo library not available",
+        exception = exceptions.OperationalError
+    )
+    return pymongo
