@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -39,7 +39,9 @@ __license__ = "Apache License, Version 2.0"
 
 import json
 
+from . import util
 from . import config
+from . import exceptions
 
 try: import redis
 except: redis = None
@@ -63,7 +65,7 @@ class Redis(object):
         url_c = config.conf("REDISTOGO_URL", None)
         url_c = config.conf("REDIS_URL", url_c)
         url = url or self.url or url_c or URL
-        self._connection = redis.from_url(url)
+        self._connection = _redis().from_url(url)
         return self._connection
 
 def get_connection(url = URL):
@@ -71,8 +73,16 @@ def get_connection(url = URL):
     if connection: return connection
     url = config.conf("REDISTOGO_URL", url)
     url = config.conf("REDIS_URL", url)
-    connection = redis.from_url(url)
+    connection = _redis().from_url(url)
     return connection
 
 def dumps(*args):
     return json.dumps(*args)
+
+def _redis(verify = True):
+    if verify: util.verify(
+        not redis == None,
+        message = "redis library not available",
+        exception = exceptions.OperationalError
+    )
+    return redis
