@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -118,6 +118,7 @@ class Request(object):
         query = "",
         scheme = None,
         address = None,
+        protocol = None,
         params = {},
         data_j = {},
         environ = {},
@@ -130,6 +131,7 @@ class Request(object):
         self.query = query
         self.scheme = scheme
         self.address = address
+        self.protocol = protocol
         self.params = params
         self.data_j = data_j
         self.environ = environ
@@ -147,7 +149,9 @@ class Request(object):
         self.authorization = None
         self.data = None
         self.result = None
+        self.result_l = None
         self.locale = None
+        self.language = None
         self.query_s = None
         self.prefixes = None
         self.session = session.MockSession(self)
@@ -185,6 +189,7 @@ class Request(object):
         self.query = None
         self.scheme = None
         self.address = None
+        self.protocol = None
         self.params = None
         self.data_j = None
         self.environ = None
@@ -200,7 +205,9 @@ class Request(object):
         self.authorization = None
         self.data = None
         self.result = None
+        self.result_l = None
         self.locale = None
+        self.language = None
         self.query_s = None
         self.prefixes = None
         self.session = None
@@ -338,6 +345,15 @@ class Request(object):
     def set_code(self, code):
         self.code = code
 
+    def get_encoded(self, encoding = None, safe = True):
+        if encoding == None: encoding = self.get_encoding()
+        if not encoding: return self.data
+        try:
+            return legacy.u(self.data, encoding = encoding, force = True)
+        except UnicodeDecodeError:
+            if not safe: raise
+            return self.data
+
     def extend_args(self, args):
         is_dict = isinstance(args, dict)
         args = args.items() if is_dict else args
@@ -516,15 +532,17 @@ class Request(object):
         value = value or self.locale
         return value.replace("-", "_").lower()
 
-    def load_locale(self, available, fallback = "en_us"):
+    def load_locale(self, available, fallback = "en_us", ensure = True):
         # tries to gather the best locale value using the currently
         # available strategies and in case the retrieved local is part
         # of the valid locales for the app returns the locale, otherwise
-        # returns the fallback value instead
+        # returns the fallback value instead (using the first available
+        # locale in case the ensure flag is set)
         locale = self.get_locale(fallback = fallback)
         locale = self.locale_b(locale)
-        if locale in available: self.locale = locale
-        else: self.locale = fallback
+        if locale in available: self.set_locale(locale)
+        elif ensure and available: self.set_locale(available[0])
+        else: self.set_locale(fallback)
 
     def get_locale(self, fallback = "en_us"):
         # tries to retrieve the locale value from the provided URL
@@ -553,6 +571,12 @@ class Request(object):
         # retrieval have failed and so the fallback value is returned
         return fallback
 
+    def set_locale(self, locale):
+        # sets both the base locale value but also the language attribute
+        # that may be used to determine the base (less specific) language value
+        self.locale = locale
+        self.language = locale.split("_", 1)[0]
+
     def get_langs(self):
         # gathers the value of the accept language header and in case
         # it's not defined returns immediately as no language can be
@@ -658,20 +682,39 @@ class Request(object):
     def get_encoding(self):
         return "utf-8"
 
+    def get_sdate(self, format = "%d/%b/%Y:%H:%M:%S +0000"):
+        sdate = datetime.datetime.utcfromtimestamp(self.stime)
+        return sdate.strftime(format)
+
+    def get_edate(self, format = "%d/%b/%Y:%H:%M:%S +0000"):
+        edate = datetime.datetime.utcfromtimestamp(self.etime)
+        return edate.strftime(format)
+
     def is_mobile(self):
-        user_agent = self.get_header("User-Agent", None)
+        user_agent = self.get_header("User-Agent")
         return util.is_mobile(user_agent)
 
     def is_tablet(self):
-        user_agent = self.get_header("User-Agent", None)
+        user_agent = self.get_header("User-Agent")
         return util.is_tablet(user_agent)
 
+    def is_browser(self):
+        user_agent = self.get_header("User-Agent")
+        return util.is_browser(user_agent)
+
+    def is_bot(self):
+        user_agent = self.get_header("User-Agent")
+        return util.is_bot(user_agent)
+
     def is_success(self):
         return self.code == 200
 
     def is_empty(self):
         return self.code in (204, 304)
 
+    def is_mock(self):
+        return False
+
     @property
     def location_f(self, safe = True):
         query = self.query_s if safe else self.query
@@ -696,13 +739,18 @@ class Request(object):
         return duration * 1000.0
 
     @property
-    def async(self):
+    def asynchronous(self):
         return True if self.get_header("X-Async") else False
 
     @property
     def partial(self):
         return True if self.get_header("X-Partial") else False
 
+    @property
+    def browser_info(self):
+        user_agent = self.get_header("User-Agent")
+        return util.browser_info(user_agent)
+
     def _resolve_p(self, params):
         secret = self.session.get("secret", None)
         if not secret: return params
@@ -713,14 +761,20 @@ class MockRequest(Request):
     Mock request class, that is meant to be used for situations
     where no web oriented request is possible to be retried or
     the logic being running outside of web request.
+
+    It's recommended to keep only one instance of a mock request
+    per each execution life-cycle.
     """
 
     def __init__(self, locale = "en_us", *args, **kwargs):
         Request.__init__(self, *args, **kwargs)
-        self.locale = locale
         self.files_s = dict()
         self.post_s = dict()
         self.params_s = dict()
+        self.set_locale(locale)
 
     def close(self):
         pass
+
+    def is_mock(self):
+        return True
