@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -474,6 +474,9 @@ class FileSession(DataSession):
         base_path = config.conf("APPIER_BASE_PATH", "")
         base_path = config.conf("SESSION_FILE_PATH", base_path)
         if base_path and not os.path.exists(base_path): os.makedirs(base_path)
+        base_path = os.path.expanduser(base_path)
+        base_path = os.path.abspath(base_path)
+        base_path = os.path.normpath(base_path)
         file_path = os.path.join(base_path, file_path)
         cls.SHELVE = shelve.open(
             file_path,
