@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -74,6 +74,18 @@ class BaseTest(unittest.TestCase):
         self.assertNotEqual(result, "Hola")
         self.assertEqual(result, "Hello")
 
+        result = self.app.has_locale("hello", locale = "pt_pt")
+        self.assertEqual(result, True)
+
+        result = self.app.has_locale("Hello", locale = "pt_pt")
+        self.assertEqual(result, False)
+
+        result = self.app.has_locale("hello", locale = "es_es")
+        self.assertEqual(result, False)
+
+        result = self.app.has_locale("Hello", locale = "es_es")
+        self.assertEqual(result, False)
+
         self.app._register_bundle(dict(hello = "Hola"), "es_es")
 
         result = self.app.to_locale("hello", locale = "es_es")
@@ -108,6 +120,18 @@ class BaseTest(unittest.TestCase):
         result = self.app.to_locale("bye", locale = "pt_pt", fallback = False)
         self.assertEqual(result, "bye")
 
+        result = self.app.has_locale("bye", locale = "en_us")
+        self.assertEqual(result, True)
+
+        result = self.app.has_locale("Bye", locale = "en_us")
+        self.assertEqual(result, False)
+
+        result = self.app.has_locale("bye", locale = "pt_pt")
+        self.assertEqual(result, False)
+
+        result = self.app.has_locale("Bye", locale = "pt_pt")
+        self.assertEqual(result, False)
+
     def test_field(self):
         request = appier.Request("GET", "/")
         request.set_params(
