@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -131,3 +131,26 @@ class RequestTest(unittest.TestCase):
         self.assertEqual(request.get_header("X-Real-IP", normalize = False), None)
         self.assertEqual(request.get_header("x-real-ip", normalize = False), None)
         self.assertEqual(request.get_header("X-REAL-IP", normalize = False), None)
+
+    def test_data(self):
+        request = appier.Request("GET", "/")
+        request.set_data(appier.legacy.bytes("hello world", encoding = "utf-8"))
+
+        self.assertEqual(type(request.get_data()), bytes)
+        self.assertEqual(request.get_data(), appier.legacy.bytes("hello world", encoding = "utf-8"))
+        self.assertEqual(type(request.get_encoded()), appier.legacy.UNICODE)
+        self.assertEqual(request.get_encoded(), appier.legacy.u("hello world"))
+
+        request = appier.Request("GET", "/")
+        request.set_data(appier.legacy.bytes("你好世界", encoding = "utf-8"))
+
+        self.assertEqual(type(request.get_data()), bytes)
+        self.assertEqual(request.get_data(), appier.legacy.bytes("你好世界", encoding = "utf-8"))
+        self.assertEqual(type(request.get_encoded()), appier.legacy.UNICODE)
+        self.assertEqual(request.get_encoded(), appier.legacy.u("你好世界"))
+        self.assertEqual(type(request.get_encoded(encoding = "ascii")), bytes)
+        self.assertEqual(request.get_data(), appier.legacy.bytes("你好世界", encoding = "utf-8"))
+        self.assertRaises(
+            UnicodeDecodeError,
+            lambda: request.get_encoded(encoding = "ascii", safe = False)
+        )
