@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2017 Hive Solutions Lda.
+# Copyright (c) 2008-2018 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -166,7 +166,7 @@ def is_iterable(object):
 
 def is_mobile(user_agent):
     """
-    Verifies if the provided user agent string represent a
+    Verifies if the provided user agent string represents a
     mobile agent, for that a series of regular expressions
     are matched against the user agent string.
 
@@ -187,7 +187,7 @@ def is_mobile(user_agent):
 
 def is_tablet(user_agent):
     """
-    Verifies if the provided user agent string represent a
+    Verifies if the provided user agent string represents a
     tablet agent, for that a series of regular expressions
     are matched against the user agent string.
 
@@ -206,6 +206,99 @@ def is_tablet(user_agent):
     is_tablet = True if tablet or mobile_prefix else False
     return is_tablet
 
+def is_browser(user_agent):
+    """
+    Verifies if the provided user agent string represents a
+    browser (interactive) agent, for that a series of verifications
+    are going to be performed against the user agent string.
+
+    :type user_agent: String
+    :param user_agent: The string containing the user agent
+    value that is going to be verified for browser presence.
+    :rtype: bool
+    :return: If the provided user agent string represents an
+    interactive browser or not.
+    """
+
+    info = browser_info(user_agent)
+    if not info: return False
+    interactive = info.get("interactive", False)
+    if not interactive: return False
+    return True
+
+def is_bot(user_agent):
+    """
+    Verifies if the provided user agent string represents a
+    bot (automated) agent, for that a series of verifications
+    are going to be performed against the user agent string.
+
+    :type user_agent: String
+    :param user_agent: The string containing the user agent
+    value that is going to be verified for bot presence.
+    :rtype: bool
+    :return: If the provided user agent string represents an
+    automated bot or not.
+    """
+
+    info = browser_info(user_agent = user_agent)
+    if not info: return False
+    bot = info.get("bot", False)
+    if not bot: return False
+    return True
+
+def browser_info(user_agent):
+    """
+    Retrieves a dictionary containing information about the browser
+    and the operative system associated with the provided user agent.
+
+    The retrieval of the information depends on the kind of user
+    agent string provided, as coverage is limited.
+
+    :type user_agent: String
+    :param user_agent: The HTTP based user agent string to be processed.
+    :rtype: Dictionary
+    :return: The dictionary/map containing the information processed from
+    the provided user agent.
+    """
+
+    info = dict()
+
+    for browser_i in defines.BROWSER_INFO:
+        identity = browser_i["identity"]
+        sub_string = browser_i.get("sub_string", identity)
+        version_search = browser_i.get("version_search", sub_string + "/")
+        interactive = browser_i.get("interactive", True)
+        bot = browser_i.get("bot", False)
+
+        if not sub_string in user_agent: continue
+        if not version_search in user_agent: continue
+
+        version_i = user_agent.index(version_search) + len(version_search)
+        version = user_agent[version_i:].split(" ", 1)[0].strip(" ;")
+        version_f = float(".".join(version.split(".")[:2]))
+        version_i = int(version_f)
+
+        info.update(
+            name = identity,
+            version = version,
+            version_f = version_f,
+            version_i = version_i,
+            interactive = interactive,
+            bot = bot
+        )
+        break
+
+    for os_i in defines.OS_INFO:
+        identity = os_i["identity"]
+        sub_string = os_i.get("sub_string", identity)
+
+        if not sub_string in user_agent: continue
+
+        info.update(os = identity)
+        break
+
+    return info if info else None
+
 def email_parts(base, encoding = None):
     """
     Unpacks the complete set of parts (name and email) from the
@@ -338,6 +431,34 @@ def request_json(request = None, encoding = "utf-8"):
     # may be used as the parsed value (post information)
     return data_j
 
+def get_context(self):
+    """
+    Retrieves the "best" possible context object for the current
+    execution life-cycle, typically this should be an "attached"
+    request object.
+
+    Multiple strategies should be used while trying to retrieved
+    the "current" context.
+    """
+
+    # tries to retrieve the request attached to the current instance
+    # (typically a property) and verifies the object compliance,
+    # returning the object to the caller in case it's valid
+    if hasattr(self, "request"):
+        request = self.request
+        is_valid = hasattr(request, "is_mock") and not request.is_mock()
+        if is_valid: return request
+
+    # uses the global strategy to try to retrieve a request for the
+    # current execution environment (not thread safe)
+    request = common.base().get_request()
+    is_valid = hasattr(request, "is_mock") and not request.is_mock()
+    if is_valid: return request
+
+    # fallback return value meaning that it was not possible to retrieve
+    # any valid execution context for the current environment
+    return None
+
 def get_object(
     object = None,
     alias = False,
@@ -1020,6 +1141,7 @@ def parse_multipart(data, boundary):
     """
 
     ordered = []
+    ordered_m = dict()
     post = dict()
     files = dict()
 
@@ -1032,8 +1154,15 @@ def parse_multipart(data, boundary):
     parts = data.split(boundary_value)
     parts[-1] = parts[-1][:boundary_extra_l * -1]
 
+    # iterates over the complete set of parts in the multi part payload
+    # to process them and add them to the appropriate dictionary and list
     for part in parts:
+        # in case the current part is not valid or empty skips the
+        # current cycle (nothing to be done)
         if not part: continue
+
+        # splits the current part around the beginning of part sequence
+        # and retrieves the proper contents if they exist
         part_s = part.split(b"\r\n\r\n", 1)
         headers = part_s[0]
         if len(part_s) > 1: contents = part_s[1]
@@ -1114,12 +1243,20 @@ def parse_multipart(data, boundary):
             target = post
             value = contents if contents == None else contents.decode("utf-8")
 
+        exists = name in ordered_m
+
         sequence = target.get(name, [])
         sequence.append(value)
-        tuple_s = (name, sequence)
-        exists = name in target
         target[name] = sequence
-        if not exists: ordered.append(tuple_s)
+
+        sequence_o = ordered_m.get(name, [])
+        sequence_o.append(value)
+        ordered_m[name] = sequence_o
+
+        if exists: continue
+
+        tuple_s = (name, sequence_o)
+        ordered.append(tuple_s)
 
     return (post, files, ordered)
 
@@ -1360,11 +1497,64 @@ def to_tokens_m(tokens):
     # method so that it may be used for structure verification
     return tokens_m
 
-def dict_merge(first, second, override = True):
+def dict_merge(first, second, override = True, recursive = False):
+    # in case no override exists then the order of the items is
+    # exchanged so that the first overrides the second values
+    # and not the exact opposite
     if not override: first, second = second, first
-    final = dict(first)
-    final.update(second)
-    return final
+
+    # in case the recursive flag is set, must iterate over all
+    # of the first items to try to merge any possible dictionary
+    # value using a recursive strategy
+    if recursive:
+        # creates the dictionary that is going to store the final
+        # merged value resulting from both dictionaries
+        final = dict()
+
+        # runs the main iteration cycles around the first dictionary
+        # trying to find possible conflicts that would required a
+        # smarter merge strategy
+        for key, value in legacy.iteritems(first):
+            # in case the current key is not present in the second
+            # dictionary (there's no conflict) and so a simple set
+            # strategy should be applied
+            if not key in second:
+                final[key] = value
+                continue
+
+            # retrieves the other (second) value and determines if
+            # it represents a dictionary (smart merge) or if instead
+            # it's of any other type (no smart merge possible)
+            other = second[key]
+            if isinstance(other, dict):
+                final[key] = dict_merge(
+                    value,
+                    other,
+                    override = override,
+                    recursive = recursive
+                )
+            else:
+                final[key] = other
+
+        # runs the final iteration cycles around the second dictionary
+        # values to try to set the unique second values in the final
+        for key, value in legacy.iteritems(second):
+            if key in final: continue
+            final[key] = value
+
+        # returns the final merged result to the caller method, this
+        # result should contain all of its dictionary values properly
+        # merged within both the first and second values
+        return final
+
+    # otherwise (uses a simple strategy) and creates a new dictionary
+    # for the first value, then updates it with the second set of
+    # dictionary values, returning then the newly created dictionary
+    # to the caller method (basic update strategy)
+    else:
+        final = dict(first)
+        final.update(second)
+        return final
 
 def cached(function):
 
@@ -1372,11 +1562,23 @@ def cached(function):
 
     @functools.wraps(function)
     def _cached(self, *args, **kwargs):
-        properties = self.request.properties
-        exists = name in properties
+        # tries to retrieve the current execution context, most
+        # of the times this should be a request object for the
+        # current temporary execution life-cycle
+        context = get_context(self)
+
+        # retrieves the properties map (if possible) and then
+        # verifies the existence or not of the name in such map
+        # returning the value immediately if it's cached
+        properties = context.properties if context else None
+        exists = name in properties if properties else False
         if exists: return properties[name]
+
+        # as no cache retrieval was possible executes the function
+        # operation and caches the resulting value into the properties
+        # map (in case it exists)
         value = function(self, *args, **kwargs)
-        properties[name] = value
+        if not properties == None: properties[name] = value
         return value
 
     return _cached
@@ -1422,7 +1624,7 @@ def delayed(function):
 
     return _delayed
 
-def route(url, method = "GET", async = False, json = False):
+def route(url, method = "GET", asynchronous = False, json = False):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
@@ -1430,14 +1632,14 @@ def route(url, method = "GET", async = False, json = False):
             method,
             url,
             function,
-            async = async,
+            asynchronous = asynchronous,
             json = json
         )
         return function
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        route = (url, method, async, json)
+        route = (url, method, asynchronous, json)
         if not hasattr(function, "_routes"): function._routes = []
         function._routes.append(route)
         function.creation_counter = CREATION_COUNTER
@@ -1570,6 +1772,24 @@ def verify(condition, message = None, code = None, exception = None):
     if not code == None: kwargs["code"] = code
     raise exception(**kwargs)
 
+def verify_equal(first, second, message = None, code = None, exception = None):
+    message = message or "Expected %s got %s" % (repr(second), repr(first))
+    return verify(
+        first == second,
+        message = message,
+        code = code,
+        exception = exception
+    )
+
+def verify_not_equal(first, second, message = None, code = None, exception = None):
+    message = message or "Expected %s not equal to %s" % (repr(first), repr(second))
+    return verify(
+        not first == second,
+        message = message,
+        code = code,
+        exception = exception
+    )
+
 def execute(args, command = None, path = None, shell = None, encoding = None):
     if shell == None: shell = os.name == "nt"
     if not encoding: encoding = sys.getfilesystemencoding()
@@ -1617,10 +1837,15 @@ class FileTuple(tuple):
     typical python class interface.
     """
 
+    @classmethod
+    def from_data(cls, data, name = None, mime = None):
+        file_tuple = cls((name, mime, data))
+        return file_tuple
+
     @classmethod
     def from_file(cls, file, name = None, mime = None):
         data = file.read()
-        file_tuple = cls((name, mime, data))
+        file_tuple = cls.from_data(data, name = name, mime = mime)
         return file_tuple
 
     @classmethod
@@ -1655,6 +1880,10 @@ class FileTuple(tuple):
     def mime(self):
         return self[1]
 
+    @property
+    def data(self):
+        return self[2]
+
 class BaseThread(threading.Thread):
     """
     The top level thread class that is meant to encapsulate
